package com.imcode.forum.shell;

import com.imcode.forum.ForumRepository;
import com.imcode.forum.sql.SqlRepository;
import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.UnhandledException;
import org.apache.commons.lang.StringUtils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ForumShell {

    private ForumRepository repository;
    private PrintStream out;
    private BufferedReader in;

    private ForumShell(ForumRepository repository, PrintStream out) {
        this.repository = repository;
        this.out = out;
        in = new BufferedReader(new InputStreamReader(System.in));
    }

    private static BasicDataSource createDataSource(String[] args) {
        BasicDataSource dataSource = new BasicDataSource();
        dataSource.setDriverClassName(args[0]);
        dataSource.setUrl(args[1]);
        dataSource.setUsername(args[2]);
        if (args.length > 3) {
            dataSource.setPassword(args[3]);
        }
        return dataSource;
    }

    public static void main(String[] args) {
        BasicDataSource dataSource = createDataSource(args);

        ForumRepository repository = new SqlRepository(dataSource);

        ForumShell forumShell = new ForumShell(repository, System.out);

        forumShell.run();
    }

    private String prompt(String prompt) {
        out.print(prompt + "> ");
        try {
            return in.readLine();
        } catch ( IOException e ) {
            throw new UnhandledException(e);
        }
    }

    private String prompt() {
        return prompt("");
    }

    private void run() {
        while ( true ) {
            parseCommand(prompt());
        }
    }

    private void parseCommand(String inputString) {
        Pattern commandPattern = Pattern.compile("^(\\w+) *");
        Matcher matcher = commandPattern.matcher(inputString);
        if ( !matcher.find() ) {
            printAvailableCommands();
            return;
        }
        String command = matcher.group(1);
        String commandParameters = matcher.replaceFirst("");
        String[] splitCommandParameters = commandParameters.split(" +");
        if (0 == splitCommandParameters[0].length()) {
            splitCommandParameters = new String[0];
        }
        handleCommand(command, splitCommandParameters);
    }

    private Map commandMap = ArrayUtils.toMap(new Object[][] {
            {"q", new QuitCommand()},
            {"lf", new ListForumsCommand()},
            {"lt", new ListThreadsCommand()},
            {"lp", new ListPostsCommand()},
            {"cf", new CreateForumCommand()},
            {"ct", new CreateThreadCommand()},
            {"cp", new CreatePostCommand()},
    });
    
    private void handleCommand(String commandName, String[] commandParameters) {
        ForumShellCommand command = (ForumShellCommand) commandMap.get(commandName) ; 
        if (null == command) {
            printAvailableCommands();
            return ;
        }
        String[] parameterNames = command.getParameterNames();
        if ( commandParameters.length != parameterNames.length ) {
            printCommandSyntax(commandName, command);
            return ;
        }
        try {
            command.execute(repository, out, commandParameters);
        } catch(NumberFormatException nfe) {
            printCommandSyntax(commandName, command);
        }
    }

    private void printCommandSyntax(String commandName, ForumShellCommand command) {
        out.println(commandName+": "+command.getPurpose()+".") ;
        out.println("Syntax: "+getCommandSyntax(commandName, command));
    }

    private void printAvailableCommands() {
        out.println("Available commands are:") ;
        SortedSet commandNames = new TreeSet(commandMap.keySet()) ;
        for ( Iterator iterator = commandNames.iterator(); iterator.hasNext(); ) {
            String commandName = (String)iterator.next();
            ForumShellCommand command = (ForumShellCommand)commandMap.get(commandName);
            out.println(StringUtils.rightPad(command.getPurpose()+":", 15)+" "+getCommandSyntax(commandName, command));
        }
    }

    private String getCommandSyntax(String commandName, ForumShellCommand command) {
        String[] parameterNames = command.getParameterNames();
        for ( int i = 0; i < parameterNames.length; i++ ) {
            parameterNames[i] = "<"+parameterNames[i]+">";
        }
        String commandSyntax = commandName + " " + StringUtils.join(parameterNames, " ");
        return commandSyntax;
    }

}
